@@ -144,6 +144,12 @@ class LocalDataset(object):
         self.annotations_path += dataset.annotations_path
         return self
 
+    def get_image(self, index: int):
+        return load_pil_image(self.images_path[index])
+
+    def get_image_path(self, index: int):
+        return self.images_path[index]
+
     def parse_json(self, index: int):
         """
         Load an annotation and filter out the extra classes according to what
