@@ -4,12 +4,11 @@ from pathlib import Path
 from typing import Generator, List
 
 import numpy as np
-from upolygon import draw_polygon
+from upolygon import draw_polygon, rle_encode
 
 import darwin.datatypes as dt
 from darwin.utils import convert_polygons_to_sequences
 
-
 class NumpyEncoder(json.JSONEncoder):
     def default(self, obj):
         if isinstance(obj, np.integer):
@@ -121,7 +120,7 @@ def build_annotations(annotation_files, categories):
 def build_annotation(annotation_file, annotation_id, annotation: dt.Annotation, categories):
     annotation_type = annotation.annotation_class.annotation_type
     if annotation_type == "polygon":
-        sequences = convert_polygons_to_sequences(annotation.data["path"])
+        sequences = convert_polygons_to_sequences(annotation.data["path"], rounding=False)
         x_coords = [s[0::2] for s in sequences]
         y_coords = [s[1::2] for s in sequences]
         min_x = np.min([np.min(x_coord) for x_coord in x_coords])
@@ -147,7 +146,7 @@ def build_annotation(annotation_file, annotation_id, annotation: dt.Annotation,
         mask = np.zeros((annotation_file.image_height, annotation_file.image_width))
         sequences = convert_polygons_to_sequences(annotation.data["paths"])
         draw_polygon(mask, sequences, 1)
-        counts = rle_encoding(mask)
+        counts = rle_encode(mask)
 
         x_coords = [s[0::2] for s in sequences]
         y_coords = [s[1::2] for s in sequences]
@@ -219,19 +218,3 @@ def polygon_area(x: np.ndarray, y: np.ndarray) -> float:
     for x and y coordinates.
     """
     return 0.5 * np.abs(np.dot(x, np.roll(y, 1)) - np.dot(y, np.roll(x, 1)))
-
-
-def rle_encoding(binary_mask):
-    counts = []
-
-    last_elem = 0
-    running_length = 0
-    for i, elem in enumerate(binary_mask.ravel(order="F")):
-        if elem != last_elem:
-            counts.append(running_length)
-            running_length = 0
-            last_elem = elem
-        running_length += 1
-
-    counts.append(running_length)
-    return counts
