@@ -119,7 +119,9 @@ class Options(object):
             "--only-annotations", action="store_true", help="Download only annotations and no corresponding images"
         )
         parser_pull.add_argument("--folders", action="store_true", help="Recreates image folders")
-        parser_pull.add_argument("--video-frames", action="store_true", help="Pulls video frame images instead of video files")
+        parser_pull.add_argument(
+            "--video-frames", action="store_true", help="Pulls video frame images instead of video files"
+        )
 
         # Import
         parser_import = dataset_action.add_parser("import", help="Import data to an existing (remote) dataset.")
