@@ -2,7 +2,7 @@ from darwin.exceptions import NameTaken, ValidationError
 
 
 def name_taken(code, body):
-    if code == 422 and body["errors"]["name"][0] == "has already been taken":
+    if code == 422 and body["errors"]["metadata"][0] == "has already been taken":
         raise NameTaken
 
 
